(function() {
  var argon, fs, out, path, _;
  fs = require('fs');
  path = require('path');
  _ = require('underscore');
  argon = require('argon');
  out = (require('styout')).instance('octo.io');
  exports.scan = function(root) {
    var callbacks, options, pending, recurse, roots, statCallback, _ref, _ref2;
    _ref = argon.ize(arguments, {
      name: 'scan',
      requiredArgs: ['file/dir'],
      events: ['dirFound', 'fileFound', 'scanComplete'],
      defaultOptions: {
        ignoreFiles: {}
      }
    }), roots = _ref[0], _ref2 = _ref[1], options = _ref2.options, callbacks = _ref2.callbacks;
    pending = {};
    if (_.last(root) === '/') {
      recurse = false;
      root = root.slice(0, -1);
    } else {
      recurse = true;
    }
    statCallback = function(relPath) {
      var filePath;
      filePath = path.join(root, relPath);
      pending[filePath] = 1;
      return function(err, stats) {
        if (err) {
          if (relPath === '.') {
            out.warn("<b>" + root + "</b> does not exist");
          }
          delete pending[filePath];
          if (_.isEmpty(pending)) {
            return typeof callbacks.scanComplete === "function" ? callbacks.scanComplete({
              root: root,
              options: options
            }) : void 0;
          }
        } else if (stats.isDirectory()) {
          return fs.readdir(filePath, function(err, files) {
            var file, fileRelPath, _i, _len;
            if (err) {
              out.debug("Error in readDir callback: <b>" + err + "</b>");
            } else {
              out.debug("dirFound: " + filePath);
              if (typeof callbacks.dirFound === "function") {
                callbacks.dirFound(filePath, {
                  root: root,
                  relPath: relPath,
                  options: options
                });
              }
              if (recurse || relPath === '.') {
                for (_i = 0, _len = files.length; _i < _len; _i++) {
                  file = files[_i];
                  fileRelPath = path.join(relPath, file);
                  if (!options.ignoreFiles[path.join(root, fileRelPath)]) {
                    fs.lstat(path.join(root, fileRelPath), statCallback(fileRelPath));
                  }
                }
              }
            }
            delete pending[filePath];
            if (_.isEmpty(pending) && _.isEmpty(files)) {
              return typeof callbacks.scanComplete === "function" ? callbacks.scanComplete({
                root: root,
                options: options
              }) : void 0;
            }
          });
        } else {
          return fs.readFile(filePath, 'utf8', function(err, data) {
            if (err) {
              out.debug("Error in readFile callback: <b>" + err + "</b>");
            } else {
              out.debug("fileFound: " + filePath);
              if (typeof callbacks.fileFound === "function") {
                callbacks.fileFound(filePath, {
                  root: root,
                  relPath: relPath,
                  options: options
                });
              }
            }
            delete pending[filePath];
            if (_.isEmpty(pending)) {
              return typeof callbacks.scanComplete === "function" ? callbacks.scanComplete({
                root: root,
                options: options
              }) : void 0;
            }
          });
        }
      };
    };
    if (!options.ignoreFiles[root]) {
      return fs.lstat(root, statCallback('.'));
    }
  };
}).call(this);
